-- Class: GameSprite
-- SDK: Corona - 09/12/11
-- Author: Andrew Burch
-- Date: 09/03/2012
-- Site: http://www.newhighscore.net
-- Contact: andrew.burch@newhighscore.net
-- Note: Maintain data on a single sprite

local tablex = require('core.tablex')

local abs = math.abs

local GameSprite = {}
local GameSprite_mt = {__index = GameSprite}

function GameSprite.new(params)
	local class = {
		x = 0,
		y = 0,
		isMoving = false,
		paused = false,
		movementSpeed = 0,
		movementPath = {},
	}
	
	return setmetatable(class, GameSprite_mt)
end

function GameSprite:initialise(params)
	local spriteId = params.spriteId
	local spriteSetId = params.spriteSetId
	local spriteFactory = params.spriteFactory
	local spriteInstance = spriteFactory:createInstance(spriteId, spriteSetId)
	
	spriteInstance.x = params.x
	spriteInstance.y = params.y
	
	self.x = params.x
	self.y = params.y
	self.spriteInstance = spriteInstance
	self.movementSpeed = params.movementSpeed
end

function GameSprite:moveToTarget(newX, newY)
	local currentX = self.x
	local currentY = self.y
	local xd = newX - currentX
	local yd = newY - currentY
	
	self.movementPath = {
		{
			axis = 'x',
			directionFactor = xd > 0 and 1 or -1,
			walkSequence = xd > 0 and 'walkright' or 'walkleft' 
		},
		{
			axis = 'y',
			directionFactor = yd > 0 and 1 or -1,
			walkSequence = yd > 0 and 'walkdown' or 'walkup' 
		},
	}

	local spriteInstance = self.spriteInstance
	local walkSequence = self.movementPath[1].walkSequence
	spriteInstance:prepare(walkSequence)
	spriteInstance:play()
	
	self.isMoving = true
	self.movementPathIndex = 1
	self.targetX = newX
	self.targetY = newY
end

function GameSprite:update(dt, time)
	if not self.isMoving then
		return
	end

	local spriteInstance = self.spriteInstance
	local movementPathIndex = self.movementPathIndex
	local movementPath = self.movementPath
	local pathInfo = movementPath[movementPathIndex]
	local axis = pathInfo.axis
	local directionFactor = pathInfo.directionFactor

	local movementSpeed = self.movementSpeed
	local advancePath = false
		
	if axis == 'x' then
		self.x = self.x + (movementSpeed * directionFactor)
		if abs(self.targetX - self.x) <= 1 then
			advancePath = true
		end
		spriteInstance.x = self.x
	else
		self.y = self.y + (movementSpeed * directionFactor)
		if abs(self.targetY - self.y) <= 1 then
			advancePath = true
		end
		spriteInstance.y = self.y
	end
	
	if advancePath then
		local pathSize = #movementPath
		local newMovementPathIndex = movementPathIndex + 1
		
		if newMovementPathIndex <= pathSize then
			local nextPathInfo = movementPath[newMovementPathIndex]
			local walkSequence = nextPathInfo.walkSequence
			spriteInstance:prepare(walkSequence)
			spriteInstance:play()
		else
			self.isMoving = false
			newMovementPathIndex = 0
			tablex.clear(self.movementPath)
			spriteInstance:pause()
		end
		
		self.movementPathIndex = newMovementPathIndex
	end
end

function GameSprite:dispose()
	self.spriteInstance:removeSelf()
	self.spriteInstance = nil
	self.isMoving = false
	self.paused = false
	
	tablex.clear(self.movementPath)
end

function GameSprite:pause()
	self.spriteInstance:pause()
	self.isMoving = false
	self.paused = true
end

function GameSprite:unpause()
	if self.paused then
		self.spriteInstance:play()
		self.isMoving = true
		self.paused = false
	end	
end


return GameSprite